@* Determinant. The determinant of a matrix~$A\in\reals[n][n]$ is given by
$$\det(A)=\sum_{i=j}^n(-1)^{j+1}a_{1j}\det(A_{1j}),$$ where~$A_{1j}$ is
an~$(n-1)$-by-$(n-1)$ matrix obtained by deleting the first row and~$j$th comlumn of~$A$.
Computing the determinant this way would require~$O(n!)$ operations, which is unacceptable.
Fortunately, we have~$\det(AB)=\det(A)\det(B)$, and for a upper or lower triangular
matrix~$\det(A)=\prod a_{ii}$. With this in mind we are able to compute the determinant
in~$2n^3/3$ operations via the LU decomposition: we have
$\det(A)=\det(LU)=\det(L)\det(U)=\det(U)= \prod u_{ii}$. The only
trick is that we have a {\em permutted\/} version, so we have to take into account the
number of permutations. The LU decomposition function returns the necessary information.
@c
/* Empty, waiting for |export| */
@ @(det.h@>=
#ifndef __MATH_DET__
#define __MATH_DET__ 1.00
#include <math/math.h>
#include <math/lu.h>
namespace math
{
    template<matrix_simple_template>@;
    T det(const matrix<T,structure,storage>& A)
    {
        matrix<T,unstructured,storage> aux = A;
        vector<math::index> pivots;
        T determinant=0;
        try
        {
	    determinant=math::lu::decompose(&aux,&pivots);
            math::index i=0;
            while(++i <= aux.rows())
		determinant *= aux(i,i)
        }
        catch (math::error::singular e)@+{}
        return determinant;
    }
}
#endif
