@* The gaxpy function. Our first functional is arguably the most simple. We define 
a function that computes the scalar |gaxpy| operation, that is, $f(x)=Ax+b$. This is also one of
the most useful functions for optimization -- think of~$Ax\prec b$.
@c
/* Empty, waiting for |export| */
@ @(function/gaxpy.h@>=
#ifndef __MATH_GAXPY_FUNCTION__
#define __MATH_GAXPY_FUNCTION__ 1.00
#include <math/math.h>
#include <math/algebra.h>
#include <math/functionbase.h>
namespace math
{
    @[namespace function@]
    {
	template<matrix_simple_template>@;
	class @[gaxpy : public base<T>@]
	{
	    @<Gaxpy function internal variables@>@;
	public:@;
	    @<Gaxpy function class methods@>@;
	};
    }
}
#endif
@ To compute a |gaxpy| operation we need two parameters, the~$A$ and~$b$ matrices. We assume~$b$
will be a vector most times.
@<Gaxpy function internal variables@>=
matrix<T,structure,storage> ay;
matrix<T,unstructured,dense> bee;
@ We provide a means to modify these values in two ways: by returning a reference to them
and at the time of construction.
@<Gaxpy function class methods@>=@!@:function::gaxpy}{A@>@!@:function::gaxpy}{b@>
matrix<T,structure,storage>& A(void) @+ {@+return ay;@+}
matrix<T,unstructured,dense>& b(void) @+ {@+return bee;@+}
gaxpy(const matrix<T,structure,storage>& a,const T& B):
    ay(a), bee(B) @+
{}
@ Now to the fun stuff. Evaluating is a simple task of calling some algebra functions.
@<Gaxpy function class methods@>=@!@:function::gaxpy}{eval@>
virtual matrix<T,unstructured,dense>&
eval(const matrix<T,unstructured,dense>& x, matrix<T,unstructured,dense>* dest)
{@+
    return(gaxpy(ay,x,&(dest->copyfrom(b))));@+
}
@ The Jacobian is simply~$A$.
@<Gaxpy function class methods@>=@!@:function::gaxpy}{jacobian@>
@:matrix}{cols@>@:matrix}{entry@>
virtual matrix<T,unstructured,dense>&
jacobian(const matrix<T,unstructured,dense>& x, matrix<T,unstructured,dense>* dest)
{
    return (*dest)=ay;
}
@ The hessian of an affine function is zero.
@<Gaxpy function class methods@>=@!@:function::gaxpy}{hess@>
@:matrix}{rows@>@:matrix}{init@>@:matrix}{fillwith@>
virtual matrix<T,symmetric,dense>&
hess(const matrix<T,unstructured,dense>& x, matrix<T,symmetric,dense>* dest, const index, const 
index)
{
    dest->resize(x.rows(),x.rows());@:matrix}{resize@>@:matrix}{rows@>
    dest->fillwith(0);@:matrix}{fillwith@>
    return *dest;
}
virtual matrix<T,symmetric,sparse>&
hess(const matrix<T,unstructured,dense>& x, matrix<T,symmetric,sparse>* dest, const index,
const index)
{
    *dest=@[matrix<T,symmetric,sparse>@](x.rows(),x.rows());@:matrix}{rows@>
    return *dest;
}
