@** The symmetric structure. Now that we have a new storage defined, let us end the lesson
on how to define new matrices by creating a new structure.
@c
/* Empty, waiting for |export| */
@ @(symmetric.h@>=
#ifndef __MATH_SYMMETRIC__
#define __MATH_SYMMETRIC__ 1.0

#include<math/math.h>

namespace math
{
    @<Symmetric structure definition@>@;
}
#endif
@ @<Symmetric structure definition@>=
template<class T>@;
class symmetric
{
public:@;
    @<Symmetric structure methods@>@;
};
@ We start by defining the constructor that takes the dimensions as arguments. A symmetric
matrix has to be square, and it can be represented with only~$n(n+1)/2$ elements. We will
store them as a vector as follows: if we have a matrix $$A=\left[\matrix{ a_{11} & a_{12} &
a_{13} \cr \ast & a_{22} & a_{23} \cr \ast & \ast & a_{33} } \right],$$ then we store is as
the vector $$\left[\matrix{ a_{11} & a_{12} & a_{22} & a_{13} & a_{23} & a_{33} }\right].$$
This scheme is the same one used in the \lapack{} package, so we will use it to facilitate
interfacing with it.
@<Symmetric structure methods@>=
void resize(math::index* rows, math::index* cols)
{
    if(*rows!=*cols) throw error::nonsquare();
    
    *rows=(*rows)*(*rows+1)/2;
    *cols=1;
}
symmetric(math::index* rows,math::index* cols)@+
{@+
    resize(rows,cols);@+
}
@ Next we define the |preprocess| method. After some algebra, you will see that the
index of the element~$(i,j)$ is given by $${j(j-1)\over 2}+i$$ if $j\geq i$, that is,
if the element is on the upper triangle of the matrix. In this class these are the elements
that are stored. The elements of the lower triangle are not assignable.
@<Symmetric structure methods@>= @!@:symmetric}{preprocess@>
bool preprocess(math::index* row, math::index* col) const
{
    if(*col>=*row)
    {
        *row=(*col-1)*(*col)/2+(*row);
        *col=1;
        return true; /* Assignable. */
    }
    *row=(*row-1)*(*row)/2+(*col);
    *col=1;
    return false; /* Not assignable. */
}
@ We need a copy constructor, which is empty.
@<Symmetric structure methods@>=
symmetric(const symmetric&)@+
{}
@ And we are done. I cannot help feeling proud of the flexibility of this system. In less
than two pages we defined a whole new matrix structure, and this is taken into account
documentation!
